@@ -45,7 +45,7 @@ Project Authors
 Current code lives on github: https://github.com/salimfadhley/jenkinsapi
 
 """
-
+import sys
 from jenkinsapi import (
     # Modules
     command_line,
@@ -62,3 +62,12 @@ __all__ = [
     "fingerprint", "jenkins", "jenkinsbase", "job", "node", "result_set", "result", "view"
 ]
 __docformat__ = "epytext"
+# In case of jenkinsapi is not installed in 'develop' mode
+__version__ = '99.99.99'
+try:
+    import pkg_resources
+    __version__ = pkg_resources.working_set.by_key['jenkinsapi'].version
+except ImportError:
+    pass
+except KeyError:
+    pass
