@@ -4,15 +4,18 @@ Kerberos aware Requester
 from jenkinsapi.utils.requester import Requester
 from requests_kerberos import HTTPKerberosAuth, OPTIONAL
 
-#pylint: disable=W0222
+
+# pylint: disable=W0222
 class KrbRequester(Requester):
+
     """
     A class which carries out HTTP requests with Kerberos/GSSAPI authentication.
     """
 
     def __init__(self, ssl_verify=None, baseurl=None, mutual_auth=OPTIONAL):
         """
-        :param ssl_verify: flag indicating if server certificate in HTTPS requests should be verified
+        :param ssl_verify: flag indicating if server certificate in HTTPS requests should be
+                           verified
         :param baseurl: Jenkins' base URL
         :param mutual_auth: type of mutual authentication, use one of REQUIRED, OPTIONAL or DISABLED
                             from requests_kerberos package
@@ -25,14 +28,10 @@ class KrbRequester(Requester):
         super(KrbRequester, self).__init__(**args)
         self.mutual_auth = mutual_auth
 
-    def get_request_dict(self, url, params, data, headers):
-        req_dict = Requester(
-            self,
-            url=url,
-            params=params,
-            data=data,
-            headers=headers
-        )
+    def get_request_dict(
+            self, params=None, data=None, files=None, headers=None, **kwargs):
+        req_dict = super(KrbRequester, self).get_request_dict(params=params, data=data, files=files,
+                                                              headers=headers)
         if self.mutual_auth:
             auth = HTTPKerberosAuth(self.mutual_auth)
         else:
