@@ -1,17 +1,24 @@
 """
 Module for jenkinsapi views
 """
-import urllib
+try:
+    from urllib import urlencode
+except ImportError:
+    # Python3
+    from urllib.parse import urlencode
+
 import logging
 
 from jenkinsapi.jenkinsbase import JenkinsBase
 from jenkinsapi.job import Job
+from jenkinsapi.custom_exceptions import NotFound
 
 
 log = logging.getLogger(__name__)
 
 
 class View(JenkinsBase):
+
     """
     View class
     """
@@ -49,7 +56,13 @@ class View(JenkinsBase):
         return self.get_job_dict().keys()
 
     def iteritems(self):
-        for name, url in self.get_job_dict().iteritems():
+        try:
+            it = self.get_job_dict().iteritems()
+        except AttributeError:
+            # Python3
+            it = self.get_job_dict().items()
+
+        for name, url in it:
             api_url = self.python_api_url(url)
             yield name, Job(api_url, name, self.jenkins_obj)
 
@@ -60,9 +73,7 @@ class View(JenkinsBase):
         return [a for a in self.iteritems()]
 
     def _get_jobs(self):
-        if not 'jobs' in self._data:
-            pass
-        else:
+        if 'jobs' in self._data:
             for viewdict in self._data["jobs"]:
                 yield viewdict["name"], viewdict["url"]
 
@@ -73,13 +84,13 @@ class View(JenkinsBase):
         return len(self.get_job_dict().keys())
 
     def get_job_url(self, str_job_name):
-        try:
-            job_dict = self.get_job_dict()
-            return job_dict[str_job_name]
-        except KeyError:
-            #noinspection PyUnboundLocalVariable
-            all_views = ", ".join(job_dict.keys())
-            raise KeyError("Job %s is not known - available: %s" % (str_job_name, all_views))
+        if str_job_name in self:
+            return self.get_job_dict()[str_job_name]
+        else:
+            # noinspection PyUnboundLocalVariable
+            views_jobs = ", ".join(self.get_job_dict().keys())
+            raise NotFound("Job %s is not known, available jobs"
+                           " in view are: %s" % (str_job_name, views_jobs))
 
     def get_jenkins_obj(self):
         return self.jenkins_obj
@@ -106,7 +117,9 @@ class View(JenkinsBase):
                 top_jenkins = self.get_jenkins_obj().get_jenkins_obj_from_url(
                     self.baseurl.split('view/')[0])
                 if not top_jenkins.has_job(str_job_name):
-                    log.error(msg='Job "%s" is not known to Jenkins' % str_job_name)
+                    log.error(
+                        msg='Job "%s" is not known to Jenkins' %
+                        str_job_name)
                     return False
                 else:
                     job = top_jenkins.get_job(str_job_name)
@@ -142,7 +155,7 @@ class View(JenkinsBase):
         data[job.name] = 'on'
 
         data['json'] = data.copy()
-        data = urllib.urlencode(data)
+        data = urlencode(data)
         self.get_jenkins_obj().requester.post_and_confirm_status(
             '%s/configSubmit' % self.baseurl, data=data)
         self.poll()
@@ -157,6 +170,35 @@ class View(JenkinsBase):
     def get_nested_view_dict(self):
         return dict(self._get_nested_views())
 
+    def get_config_xml_url(self):
+        return '%s/config.xml' % self.baseurl
+
+    def get_config(self):
+        """
+        Return the config.xml from the view
+        """
+        url = self.get_config_xml_url()
+        response = self.get_jenkins_obj().requester.get_and_confirm_status(url)
+        return response.text
+
+    def update_config(self, config):
+        """
+        Update the config.xml to the view
+        """
+        url = self.get_config_xml_url()
+        try:
+            if isinstance(
+                    config, unicode):  # pylint: disable=undefined-variable
+                config = str(config)
+        except NameError:
+            # Python3 already a str
+            pass
+
+        response = self.get_jenkins_obj().requester.post_url(
+            url, params={}, data=config)
+        return response.text
+
     @property
     def views(self):
-        return self.get_jenkins_obj().get_jenkins_obj_from_url(self.baseurl).views
+        return self.get_jenkins_obj().get_jenkins_obj_from_url(
+            self.baseurl).views
