@@ -10,6 +10,7 @@ log = logging.getLogger(__name__)
 
 
 class Views(object):
+
     """
     An abstraction on a Jenkins object's views
     """
@@ -17,7 +18,8 @@ class Views(object):
     NESTED_VIEW = 'hudson.plugins.nested_view.NestedView'
     MY_VIEW = 'hudson.model.MyView'
     DASHBOARD_VIEW = 'hudson.plugins.view.dashboard.Dashboard'
-    PIPELINE_VIEW = 'au.com.centrumsystems.hudson.plugin.buildpipeline.BuildPipelineView'
+    PIPELINE_VIEW = ('au.com.centrumsystems.hudson.'
+                     'plugin.buildpipeline.BuildPipelineView')
 
     def __init__(self, jenkins):
         self.jenkins = jenkins
@@ -44,6 +46,7 @@ class Views(object):
                 raise TypeError('Job %s does not exist in Jenkins' % job_name)
 
     def __getitem__(self, view_name):
+        self.jenkins.poll()
         for row in self.jenkins._data.get('views', []):
             if row['name'] == view_name:
                 return View(row['url'], row['name'], self.jenkins)
@@ -69,6 +72,7 @@ class Views(object):
         """
         Get the names of all available views
         """
+        self.jenkins.poll()
         for row in self.jenkins._data.get('views', []):
             yield row['name']
 
@@ -86,7 +90,6 @@ class Views(object):
         :return: new View obj or None if view was not created
         """
         log.info(msg='Creating "%s" view "%s"' % (view_type, view_name))
-        #url = urlparse.urljoin(self.baseurl, "user/%s/my-views/" % person) if person else self.baseurl
 
         if view_name in self:
             log.warn(msg='View "%s" already exists' % view_name)
@@ -101,6 +104,9 @@ class Views(object):
             "json": json.dumps({"name": view_name, "mode": view_type})
         }
 
-        self.jenkins.requester.post_and_confirm_status(url, data=data, headers=headers)
+        self.jenkins.requester.post_and_confirm_status(
+            url,
+            data=data,
+            headers=headers)
         self.jenkins.poll()
         return self[view_name]
